@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 '''Chemical Engineering Design Library (ChEDL). Utilities for process modeling.
-Copyright (C) 2016, 2017 Caleb Bell <Caleb.Andrew.Bell@gmail.com>
+Copyright (C) 2016, 2017, 2018 Caleb Bell <Caleb.Andrew.Bell@gmail.com>
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -20,15 +20,18 @@ LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 SOFTWARE.'''
 
+import os
+
+from . import numerics
+from . import constants
 
 from . import atmosphere
 from . import compressible
-from . import control_valve
 from . import core
+from . import friction
 from . import filters
 from . import fittings
 from . import flow_meter
-from . import friction
 from . import geometry
 from . import mixing
 from . import open_flow
@@ -37,12 +40,14 @@ from . import piping
 from . import pump
 from . import safety_valve
 from . import packed_tower
-from . import two_phase
 from . import two_phase_voidage
+from . import two_phase
 from . import drag
 from . import saltation
 from . import separator
 from . import particle_size_distribution
+from . import jet_pump
+from . import control_valve
 
 
 from .atmosphere import *
@@ -67,18 +72,21 @@ from .two_phase_voidage import *
 from .drag import *
 from .saltation import *
 from .separator import *
+from .jet_pump import *
 
 
 __all__ = ['atmosphere', 'compressible', 'control_valve', 'core', 'filters', 'fittings',
 'friction', 'geometry', 'mixing', 'open_flow', 'packed_bed', 'piping',
 'pump', 'safety_valve', 'packed_tower', 'two_phase', 'two_phase_voidage', 
-'drag', 'saltation', 'separator', 'flow_meter', 'particle_size_distribution']
+'drag', 'saltation', 'separator', 'flow_meter', 'particle_size_distribution',
+'jet_pump']
 
 __all__.extend(atmosphere.__all__)
 __all__.extend(compressible.__all__)
 __all__.extend(control_valve.__all__)
 __all__.extend(core.__all__)
 __all__.extend(filters.__all__)
+
 __all__.extend(fittings.__all__)
 __all__.extend(friction.__all__)
 __all__.extend(geometry.__all__)
@@ -96,6 +104,16 @@ __all__.extend(drag.__all__)
 __all__.extend(saltation.__all__)
 __all__.extend(separator.__all__)
 __all__.extend(particle_size_distribution.__all__)
+__all__.extend(jet_pump.__all__)
 
+submodules = [atmosphere, compressible, core, friction, filters, fittings,
+              flow_meter, geometry, mixing, open_flow, packed_bed, piping, pump,
+              safety_valve, packed_tower, two_phase_voidage, two_phase, drag,
+              saltation, separator, particle_size_distribution, jet_pump,
+              control_valve]
+
+__version__ = '0.1.79'
+
+fluids_dir = os.path.dirname(__file__)
+fluids_data_dir = os.path.join(fluids_dir, 'data')
 
-__version__ = '0.1.71'
