@@ -48,6 +48,15 @@ except ImportError: # pragma: no cover
 '''
 # is_critical_flow is broken
 
+def func_args(func):
+    '''Basic function which returns a tuple of arguments of a function or
+    method.
+    '''
+    try:
+        return tuple(inspect.getargspec(func).args)
+    except:
+        return tuple(inspect.getfullargspec(func).args)
+
 u.autoconvert_offset_to_baseunit = True
 
 
@@ -56,11 +65,14 @@ expr2 = re.compile('Returns *\n *-+\n +')
 match_sections = re.compile('\n *[A-Za-z ]+ *\n +-+')
 match_section_names = re.compile('\n *[A-Za-z]+ *\n +-+')
 variable = re.compile('[a-zA-Z_0-9]* : ')
-match_units = re.compile('\[[a-zA-Z0-9()./*^\- ]*\]')
+match_units = re.compile(r'\[[a-zA-Z0-9().\/*^\- ]*\]')
 
 
 def parse_numpydoc_variables_units(func):
-    text = func.__doc__
+    try:
+        text = func.__doc__
+    except:
+        text = ''
     if text is None:
         text = ''
     section_names = [i.replace('-', '').strip() for i in match_sections.findall(text)]
@@ -106,9 +118,15 @@ def check_args_order(func):
     Other Parameters with the input arguments of the actual function signature.
     Raises an exception if not correctly defined.
     
+    getargspec is used for Python 2.7 compatibility and is deprecated in Python
+    3.
+    
     >>> check_args_order(fluids.core.Reynolds)
     '''
-    argspec = inspect.getargspec(func)
+    try:
+        argspec = inspect.getfullargspec(func)
+    except:
+        argspec = inspect.getargspec(func)
     parsed_data = parse_numpydoc_variables_units(func)
     # compare the parsed arguments with those actually defined
     parsed_units = parsed_data['Parameters']['units']
@@ -118,12 +136,15 @@ def check_args_order(func):
         parsed_units += parsed_data['Other Parameters']['units']
     
     if argspec.args != parsed_parameters: # pragma: no cover
-        raise Exception('Function signature is not the same as the documentation'
-                        'signature = %s; documentation = %s' %(argspec.args, parsed_parameters))
+        raise Exception('Function %s signature is not the same as the documentation'
+                        ' signature = %s; documentation = %s' %(func.__name__, argspec.args, parsed_parameters))
     
     
 def match_parse_units(doc, i=-1):
-    matches = match_units.findall(doc)
+    if doc is None:
+        matches = ['[]']
+    else:
+        matches = match_units.findall(doc)
     if len(matches) == 0:
         # If there is no unit listed, assume it's dimensionless (probably a string)
         matches = ['[]']
@@ -157,6 +178,15 @@ def convert_input(val, unit, ureg, strict=True):
         else:
             return val
 
+pint_expression_cache = {}
+
+def parse_expression_cached(unit, ureg):
+    if unit in pint_expression_cache:
+        return pint_expression_cache[unit]
+    ans = ureg.parse_expression(unit)
+    pint_expression_cache[unit] = ans
+    return ans
+    
 
 def convert_output(result, out_units, out_vars, ureg):
     # Attempt to handle multiple return values
@@ -167,15 +197,15 @@ def convert_output(result, out_units, out_vars, ureg):
     elif t == dict:
         for key, ans in result.items():
             unit = out_units[out_vars.index(key)]
-            result[key] = ans*ureg.parse_expression(unit)
+            result[key] = ans*parse_expression_cached(unit, ureg)
         return result
     elif isinstance(result, collections.Iterable):
         conveted_result = []
         for ans, unit in zip(result, out_units):
-            conveted_result.append(ans*ureg.parse_expression(unit))
+            conveted_result.append(ans*parse_expression_cached(unit, ureg))
         return conveted_result
     else:
-        return result*ureg.parse_expression(out_units[0])
+        return result*parse_expression_cached(out_units[0], ureg)
 
 
 def wraps_numpydoc(ureg, strict=True):    
@@ -263,7 +293,7 @@ class UnitAwareClass(object):
         try:
             value = getattr(self.wrapped, name)
         except Exception as e:
-            raise Exception('Failed to get property %s with error %s' %(str(name), str(e)))
+            raise AttributeError('Failed to get property %s with error %s' %(str(name), str(e)))
         if value is not None:
             if name in self.property_units:
                 if type(value) == dict:
@@ -293,7 +323,7 @@ class UnitAwareClass(object):
                         return convert_output(result, out_units, out_vars, self.ureg)
                         
                     return call_func_with_inputs_to_SI
-                raise Exception('Error: Property does not yet have units attached')
+                raise AttributeError('Error: Property does not yet have units attached')
         else:
             return value
         
@@ -340,12 +370,12 @@ def wrap_numpydoc_obj(obj_to_wrap):
     for i in dir(obj_to_wrap):
         attr = getattr(obj_to_wrap, i)
         if isinstance(attr, types.FunctionType) or isinstance(attr, types.MethodType) or type(attr) == property:
-            if type(attr) == property:
+            if type(attr) is property:
                 name = attr.fget.__name__
             else:
                 name = attr.__name__
             if hasattr(attr, '__doc__'):
-                if type(attr) == property:
+                if type(attr) is property:
                     property_unit_map[name] = u(match_parse_units(attr.fget.__doc__, i=0))
                 else:
                     parsed = parse_numpydoc_variables_units(attr)
@@ -366,10 +396,10 @@ def wrap_numpydoc_obj(obj_to_wrap):
             property_unit_map.update({var:u(unit) for var, unit in zip(parsed['Parameters']['vars'], parsed['Parameters']['units'])} )
 
     name = obj_to_wrap.__name__
-    locals()[name] = type(name, (UnitAwareClass,), 
+    fun = type(name, (UnitAwareClass,), 
            {'wrapped': obj_to_wrap, #'__doc__': obj_to_wrap.__doc__,
             'property_units': property_unit_map, 'method_units': callable_methods})
-    return locals()[name]
+    return fun
 
 
 __funcs = {}
@@ -385,6 +415,10 @@ for name in dir(fluids):
         obj = wraps_numpydoc(u)(obj)
     elif type(obj) == type:
         obj = wrap_numpydoc_obj(obj)
+    elif type(obj) is types.ModuleType:
+        # Functions accessed with the namespace like friction.friction_factor
+        # would call the original function - leads to user confusion if they are exposed
+        continue
     elif isinstance(obj, str):
         continue
     if name == '__all__':
@@ -421,7 +455,6 @@ wrapped_Panhandle_A = Panhandle_A
 wrapped_Muller = Muller
 wrapped_IGT = IGT
 wrapped_nu_mu_converter = nu_mu_converter
-wrapped_SA_tank= SA_tank
 
 wrapped_differential_pressure_meter_solver = differential_pressure_meter_solver
 
@@ -433,21 +466,6 @@ def nu_mu_converter(rho, mu=None, nu=None):
     return ans*u.m**2/u.s
 
 
-def SA_tank(D, L, sideA=None, sideB=None, sideA_a=0*u.m,
-             sideB_a=0*u.m, sideA_f=None, sideA_k=None, sideB_f=None, sideB_k=None,
-             full_output=False):
-    ans = wrapped_SA_tank(D, L, sideA, sideB, sideA_a, sideB_a, sideA_f, 
-                          sideA_k, sideB_f, sideB_k, full_output)
-    if full_output:
-        SA, (sideA_SA, sideB_SA, lateral_SA) = ans
-    else:
-        SA = ans
-    if full_output:
-        return SA, (sideA_SA*u.m**2, sideB_SA*u.m**2, lateral_SA*u.m**2)
-    else:
-        return SA
-
-
 def isothermal_gas(rho, fd, P1=None, P2=None, L=None, D=None, m=None): # pragma: no cover
     '''
     >>> isothermal_gas(rho=11.3*u.kg/u.m**3, fd=0.00185*u.dimensionless, P1=1E6*u.Pa, P2=9E5*u.Pa, L=1000*u.m, D=0.5*u.m)
@@ -540,4 +558,3 @@ def differential_pressure_meter_solver(D, rho, mu, k, D2=None, P1=None, P2=None,
     elif P1 is None and (None not in [D, D2, m, P2]):
         return ans*u.Pa
 
-### TODO list
